import pytest

from django.test.client import RequestFactory

from greg.models import Identity
from gregui.authentication.auth_backends import GregOIDCBackend
from gregui.models import GregUserProfile


@pytest.mark.django_db
def test_sponsor_first_login(sponsor_foo):
    auth_request = RequestFactory().get("/foo", {"code": "foo", "state": "bar"})
    auth_request.session = {}

    backend = GregOIDCBackend()
    backend.request = auth_request

    with pytest.raises(GregUserProfile.DoesNotExist):
        GregUserProfile.objects.get(sponsor=sponsor_foo)

    claims = {
        "sub": "subsub",
        "connect-userid_sec": [f"feide:{sponsor_foo.feide_id}"],
        "dataporten-userid_sec": [f"feide:{sponsor_foo.feide_id}", "nin:12345612345"],
        "name": f"{sponsor_foo.first_name} {sponsor_foo.last_name}",
        "email": "foo@example.org",
        "email_verified": True,
        "picture": "https://foo.org/p:2192dff7-6989-4244-83cc-ae5e78875bdd",
    }

    user = backend.create_user(claims)
    user_profile = GregUserProfile.objects.get(user=user)
    assert user_profile.person is None
    sponsor = user_profile.sponsor
    assert sponsor == sponsor_foo
    assert sponsor.feide_id == sponsor_foo.feide_id


@pytest.mark.django_db
def test_sponsor_update(sponsor_foo):
    auth_request = RequestFactory().get("/foo", {"code": "foo", "state": "bar"})
    auth_request.session = {}

    backend = GregOIDCBackend()
    backend.request = auth_request

    with pytest.raises(GregUserProfile.DoesNotExist):
        GregUserProfile.objects.get(sponsor=sponsor_foo)

    assert sponsor_foo.first_name == "Sponsor"
    assert sponsor_foo.work_email == "foo@example.org"

    claims = {
        "sub": "subsub",
        "connect-userid_sec": [f"feide:{sponsor_foo.feide_id}"],
        "dataporten-userid_sec": [f"feide:{sponsor_foo.feide_id}", "nin:12345612345"],
        "name": f"Baz {sponsor_foo.last_name}",
        "email": "baz@example.org",
        "email_verified": True,
        "picture": "https://foo.org/p:2192dff7-6989-4244-83cc-ae5e78875bdd",
    }

    user = backend.create_user(claims)
    user_profile = GregUserProfile.objects.get(user=user)
    assert user_profile.person is None
    assert user_profile.sponsor == sponsor_foo

    assert user_profile.sponsor.first_name == "Baz"
    assert user_profile.sponsor.work_email == "baz@example.org"


@pytest.mark.django_db
def test_no_existing_person():
    """
    Normal login, user whitout existing person or sponsor object.

    Expect a UserProfile with no person or sponsor linked.
    """

    auth_request = RequestFactory().get("/foo", {"code": "foo", "state": "bar"})
    auth_request.session = {}
    backend = GregOIDCBackend()
    backend.request = auth_request

    claims = {
        "sub": "subsub",
        "connect-userid_sec": ["feide:foo@bar.org"],
        "dataporten-userid_sec": ["feide:foo@bar.org", "nin:12345612345"],
        "name": "Foo Ba",
        "email": "Bar",
        "email_verified": True,
        "picture": "https://foo.org/p:2192dff7-6989-4244-83cc-ae5e78875bdd",
    }

    user = backend.create_user(claims)
    user_profile = GregUserProfile.objects.get(user=user)
    assert user_profile.person is None
    assert user_profile.sponsor is None


@pytest.mark.django_db
def test_existing_person(person_foo):
    """
    Normal login, user with existing person and no sponsor object.

    Expect a UserProfile with person but not sponsor linked.
    """

    auth_request = RequestFactory().get("/foo", {"code": "foo", "state": "bar"})
    auth_request.session = {}

    backend = GregOIDCBackend()
    backend.request = auth_request

    feide_id = (
        person_foo.identities.filter(type=Identity.IdentityType.FEIDE_ID).first().value
    )

    with pytest.raises(GregUserProfile.DoesNotExist):
        GregUserProfile.objects.get(person=person_foo)

    claims = {
        "sub": "subsub",
        "connect-userid_sec": [f"feide:{feide_id}"],
        "dataporten-userid_sec": [f"feide:{feide_id}", f"nin:{person_foo.fnr.value}"],
        "name": f"{person_foo.first_name} {person_foo.last_name}",
        "email": f"{person_foo.private_email.value}",
        "email_verified": True,
        "picture": "https://foo.org/p:2192dff7-6989-4244-83cc-ae5e78875bdd",
    }

    user = backend.create_user(claims)
    user_profile = GregUserProfile.objects.get(user=user)
    assert user_profile.person == person_foo
    assert user_profile.sponsor is None


@pytest.mark.django_db
def test_invited_user(invited_person):
    """
    Invitation login, user with existing person and no sponsor object.

    Expect a UserProfile with person but not sponsor linked.
    """

    person, invitation_link = invited_person

    auth_request = RequestFactory().get("/foo", {"code": "foo", "state": "bar"})
    auth_request.session = {"invite_id": invitation_link.uuid}

    backend = GregOIDCBackend()
    backend.request = auth_request

    with pytest.raises(GregUserProfile.DoesNotExist):
        GregUserProfile.objects.get(person=person)

    feide_id = "foo@bar.com"
    nin = "12345678910"

    claims = {
        "sub": "subsub",
        "connect-userid_sec": [f"feide:{feide_id}"],
        "dataporten-userid_sec": [f"feide:{feide_id}", f"nin:{nin}"],
        "name": f"{person.first_name} {person.last_name}",
        "email": f"{feide_id}",
        "email_verified": True,
        "picture": "https://foo.org/p:2192dff7-6989-4244-83cc-ae5e78875bdd",
    }

    user = backend.create_user(claims)
    user_profile = GregUserProfile.objects.get(user=user)

    person.refresh_from_db()
    assert user_profile.person == person
    assert user_profile.sponsor is None

    person_nins = person.identities.filter(
        type=Identity.IdentityType.NORWEGIAN_NATIONAL_ID_NUMBER,
    ).all()
    assert len(person_nins) == 1
    assert person_nins[0].verified == Identity.Verified.AUTOMATIC
    assert person_nins[0].value == nin

    person_feide_ids = person.identities.filter(
        type=Identity.IdentityType.FEIDE_ID,
    ).all()
    assert len(person_feide_ids) == 1
    assert person_feide_ids[0].verified == Identity.Verified.AUTOMATIC
    assert person_feide_ids[0].value == feide_id

    person_feide_email = person.identities.filter(
        type=Identity.IdentityType.FEIDE_EMAIL
    ).all()
    assert len(person_feide_email) == 1
    assert person_feide_email[0].value == feide_id
    assert person_feide_email[0].verified == Identity.Verified.AUTOMATIC


@pytest.mark.django_db
def test_invited_user_no_id(invited_person_no_ids):
    """
    Invitation login, user with existing person and no sponsor object.

    The existing Person has no IDs, tests that we use the person
    from the invitation. Verify that the person object is updated.
    Expect a UserProfile with an updated person but not sponsor linked.
    """

    person, invitation_link = invited_person_no_ids

    auth_request = RequestFactory().get("/foo", {"code": "foo", "state": "bar"})
    auth_request.session = {"invite_id": invitation_link.uuid}

    assert (
        len(
            person.identities.filter(
                type=Identity.IdentityType.NORWEGIAN_NATIONAL_ID_NUMBER
            ).all()
        )
        == 0
    )
    assert len(person.identities.filter(type=Identity.IdentityType.FEIDE_ID).all()) == 0

    backend = GregOIDCBackend()
    backend.request = auth_request

    with pytest.raises(GregUserProfile.DoesNotExist):
        GregUserProfile.objects.get(person=person)

    feide_id = "foo@bar.org"
    nin = "12345612345"

    claims = {
        "sub": "subsub",
        "connect-userid_sec": ["feide:{feide_id}"],
        "dataporten-userid_sec": [f"feide:{feide_id}", f"nin:{nin}"],
        "name": f"{person.first_name} {person.last_name}",
        "email": f"{feide_id}",
        "email_verified": True,
        "picture": "https://foo.org/p:2192dff7-6989-4244-83cc-ae5e78875bdd",
    }

    user = backend.create_user(claims)
    user_profile = GregUserProfile.objects.get(user=user)
    assert user_profile.person == person
    assert user_profile.sponsor is None

    person_nins = person.identities.filter(
        type=Identity.IdentityType.NORWEGIAN_NATIONAL_ID_NUMBER,
    ).all()
    assert len(person_nins) == 1
    assert person_nins[0].value == nin
    assert person_nins[0].verified == Identity.Verified.AUTOMATIC

    person_feide_ids = person.identities.filter(
        type=Identity.IdentityType.FEIDE_ID
    ).all()
    assert len(person_feide_ids) == 1
    assert person_feide_ids[0].value == feide_id
    assert person_feide_ids[0].verified == Identity.Verified.AUTOMATIC

    person_feide_email = person.identities.filter(
        type=Identity.IdentityType.FEIDE_EMAIL
    ).all()
    assert len(person_feide_email) == 1
    assert person_feide_email[0].value == feide_id
    assert person_feide_email[0].verified == Identity.Verified.AUTOMATIC
